/****************************************************************************/
/*! @file
@brief DirectShow allocator wrapper for Windows media format

DirectShow̃AP[^[bvWindows Media Format SDKp̃AP[^[
-----------------------------------------------------------------------------
	Copyright (C) 2005 T.Imoto <http://www.kaede-software.com>
-----------------------------------------------------------------------------
@author		T.Imoto
@date		2005/10/28
@note
*****************************************************************************/

#include "CWMAllocator.h"
#include "CWMBuffer.h"
#include <assert.h>


//----------------------------------------------------------------------------
//! @brief	  	fXgN^
//----------------------------------------------------------------------------
CWMAllocator::~CWMAllocator()
{
}

//----------------------------------------------------------------------------
//! @brief	  	vꂽC^[tFCXԂ
//! @param		riid : C^[tFCXIID
//! @param		ppv : C^[tFCXԂ|C^[ւ̃|C^
//! @return		G[R[h
//----------------------------------------------------------------------------
STDMETHODIMP CWMAllocator::NonDelegatingQueryInterface( REFIID riid, void ** ppv )
{
	if( IID_IWMReaderAllocatorEx == riid ) {
		return GetInterface( static_cast<IWMReaderAllocatorEx*>(this), ppv );
	} else if( IID_IMemAllocator == riid ) {
		return GetInterface( static_cast<IMemAllocator*>(m_MemAlloc), ppv );
	} else {
		return CUnknown::NonDelegatingQueryInterface(riid, ppv);
	}
}
//----------------------------------------------------------------------------
//! @brief	  	obt@mۂ ( Xg[vꂽꍇɂ̃\bhĂ΂ )
//! @param		wStreamNum : Xg[ԍ
//! @param		cbBuffer : vꂽobt@TCY
//! @param		ppBuffer : obt@Ԃ|C^̃|C^
//! @param		dwFlags : prtO
//! @param		cnsSampleTime : Tv^C
//! @param		cnsSampleDuration : Tv\
//! @param		pvContext : IWMReader::StartR[ɓnꂽ|C^
//! @return		G[R[h
//----------------------------------------------------------------------------
STDMETHODIMP CWMAllocator::AllocateForStreamEx( WORD wStreamNum, DWORD cbBuffer, INSSBuffer **ppBuffer,
	DWORD dwFlags, QWORD cnsSampleTime, QWORD cnsSampleDuration, void *pvContext )
{
	HRESULT	hr;

	if( ppBuffer == NULL )
		return E_INVALIDARG;

	DWORD	flag = 0;
	if( dwFlags & WM_SFEX_NOTASYNCPOINT )	//  Key frame
		flag |= AM_GBF_NOTASYNCPOINT;

	REFERENCE_TIME	StartTime = cnsSampleTime;
	REFERENCE_TIME	EndTime = cnsSampleTime + cnsSampleDuration;

	IMediaSample	*pSample = NULL;
	hr = Allocator()->GetBuffer( &pSample, &StartTime, &EndTime, flag );
	if( hr == S_OK )
	{
		*ppBuffer = new CWMBuffer(pSample);
		(*ppBuffer)->AddRef();
		(*ppBuffer)->SetLength(cbBuffer);
		pSample->Release();	// CWMBuffer֓n_ŎQƃJEg
	}
	return hr;
}
//----------------------------------------------------------------------------
//! @brief	  	obt@mۂ ( AEgvbgvꂽꍇɂ̃\bhĂ΂ )
//! @param		wStreamNum : Xg[ԍ
//! @param		cbBuffer : vꂽobt@TCY
//! @param		ppBuffer : obt@Ԃ|C^̃|C^
//! @param		dwFlags : prtO
//! @param		cnsSampleTime : Tv^C
//! @param		cnsSampleDuration : Tv\
//! @param		pvContext : IWMReader::StartR[ɓnꂽ|C^
//! @return		G[R[h
//! @note		݂̃\bh͎gĂȂBԂ񐳏ɋ@\ȂB
//----------------------------------------------------------------------------
STDMETHODIMP CWMAllocator::AllocateForOutputEx( DWORD dwOutputNum, DWORD cbBuffer, INSSBuffer **ppBuffer,
	DWORD dwFlags, QWORD cnsSampleTime, QWORD cnsSampleDuration, void *pvContext )
{
	HRESULT	hr;

	if( ppBuffer == NULL )
		return E_INVALIDARG;

	DWORD	flag = 0;
	if( dwFlags & WM_SFEX_NOTASYNCPOINT )	//  Key frame
		flag |= AM_GBF_NOTASYNCPOINT;

	REFERENCE_TIME	StartTime = cnsSampleTime;
	REFERENCE_TIME	EndTime = cnsSampleTime + cnsSampleDuration;

	IMediaSample	*pSample;
	hr = Allocator()->GetBuffer( &pSample, &StartTime, &EndTime, flag );
	if( hr == S_OK )
	{
		*ppBuffer = new CWMBuffer(pSample);
		(*ppBuffer)->AddRef();
		pSample->Release();	// CWMBuffer֓n_ŎQƃJEg
	}
	return hr;
}
